<?php
/**
 * @version    0.0.1
 * @package    com_ganewsletter
 * @author     Glenn Arkell <glenn@glennarkell.com.au>
 * @copyright  2021 Glenn Arkell
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

namespace GlennArkell\Component\Ganewsletter\Site\Controller;

\defined('_JEXEC') or die;

use \Joomla\CMS\Application\SiteApplication;
use \Joomla\CMS\Factory;
use \Joomla\CMS\Language\Multilanguage;
use \Joomla\CMS\Language\Text;
use \Joomla\CMS\MVC\Controller\BaseController;
use \Joomla\CMS\Router\Route;
use \Joomla\CMS\Uri\Uri;
use \Joomla\Utilities\ArrayHelper;
use \GlennArkell\Component\Ganewsletter\Administrator\Helper\GanewsletterHelper;

/**
 * Item class.
 * @since  1.6.0
 */
class NewsletterController extends BaseController
{
	/**
	 * Method to check out an item for editing and redirect to the edit form.
	 * @return void
	 * @since    1.6
     * @throws Exception
	 */
	public function edit()
	{
		$app = Factory::getApplication();

		// Get the previous edit id (if any) and the current edit id.
		$previousId = (int) $app->getUserState('com_ganewsletter.edit.newsletter.id');
		$editId     = $app->input->getInt('id', 0);

		// Set the record id to edit in the session.
		$app->setUserState('com_ganewsletter.edit.newsletter.id', $editId);

		// Get the model.
		$model = $this->getModel('Newsletter', 'Site');

		// Check out the record
		if ($editId) {
			$model->checkout($editId);
		}

		// Check in the previous record.
		if ($previousId && $previousId !== $editId) {
			$model->checkin($previousId);
		}

		// Redirect to the edit screen.
		$this->setRedirect(Route::_('index.php?option=com_ganewsletter&view=newsletterform&layout=edit', false));
	}

	/**
	 * Method to publish record
	 * @return    void
	 * @throws Exception
	 * @since    1.6
	 */
	public function publish()
	{
		// Initialise variables.
		$app = Factory::getApplication();

		// Checking if the user can action the object
		$user = GanewsletterHelper::getSpecificUser();

		if ($user->authorise('core.edit', 'com_ganewsletter') || $user->authorise('core.edit.state', 'com_ganewsletter'))
		{
			$model = $this->getModel('Newsletter', 'Site');

			// Get the user data.
			$id    = $app->input->getInt('id');
			$state = $app->input->getInt('state');

			// Attempt to save the data.
			$return = $model->publish($id, $state);

			// Check for errors.
			if ($return === false)
			{
				$this->setMessage(Text::sprintf('COM_GANEWSLETTER_SAVE_FAILED', $model->getError()), 'warning');
			}

			// Clear the record id from the session.
			$app->setUserState('com_ganewsletter.edit.newsletter.id', null);

			// Flush the data from the session.
			$app->setUserState('com_ganewsletter.edit.newsletter.data', null);

			// Redirect to the list screen.
			$this->setMessage(Text::_('COM_GANEWSLETTER_ITEM_SAVED_SUCCESSFULLY'));
			$menu = Factory::getApplication()->getMenu();
			$item = $menu->getActive();

			if (!$item) {
				// If there isn't any menu item active, redirect to list view
				$this->setRedirect(Route::_('index.php?option=com_ganewsletter&view=newsletters', false));
			} else {
                $this->setRedirect(Route::_('index.php?Itemid='. $item->id, false));
			}
		} else {
			throw new \Exception(500);
		}
	}

	/**
	 * Remove data
	 * @return void
	 * @throws Exception
	 */
	public function remove()
	{
		// Initialise variables.
		$app = Factory::getApplication();

		// Checking if the user can remove object
		$user = GanewsletterHelper::getSpecificUser();

		if ($user->authorise('core.delete', 'com_ganewsletter'))
		{
			$model = $this->getModel('Newsletter', 'Site');

			// Get the user submitted data.
			$id = $app->input->getInt('id', 0);

			// Attempt to save the data.
			$return = $model->delete($id);

			// Check for errors.
			if ($return === false) {
				$this->setMessage(Text::sprintf(Text::_('COM_GANEWSLETTER_ITEM_DELETED_FAILED'), $model->getError()), 'warning');
			} else {
				// Check in the record.
				if ($return) {
					$model->checkin($return);
				}

                $app->setUserState('com_ganewsletter.edit.newsletter.id', null);
                $app->setUserState('com_ganewsletter.edit.newsletter.data', null);

                $app->enqueueMessage(Text::_('COM_GANEWSLETTER_ITEM_DELETED_SUCCESSFULLY'), 'success');
                $app->redirect(Route::_('index.php?option=com_ganewsletter&view=newsletters', false));
			}

			// Redirect to the list screen.
			$menu = Factory::getApplication()->getMenu();
			$item = $menu->getActive();
			$this->setRedirect(Route::_($item->link, false));
		} else {
			throw new \Exception(500);
		}
	}

	/**
	 * Generate PDF
	 * @return void
	 * @throws Exception
	 */
	public function genPdf()
	{
		// Initialise variables.
		$app = Factory::getApplication();

		// Get the user submitted data.
		$id = $app->input->getInt('id', 0);
		$item = GanewsletterHelper::getNewsletterRecord($id);

		// Checking if the user can remove object
		$user = GanewsletterHelper::getSpecificUser();
		$canEdit = GanewsletterHelper::canUserEdit($user, $item);

		if ($canEdit)
		{
			$model = $this->getModel('Newsletter', 'Site');

			// Attempt to save the data.
			$return = $model->genPdf($id, $item);

			// Check for errors.
			if ($return === false) {
				$this->setMessage(Text::sprintf(Text::_('COM_GANEWSLETTER_GEN_PDF_FAILED'), $model->getError()), 'warning');
			} else {
				// Check in the record.
				if ($return) {
					$model->checkin($return);
				}

                $app->setUserState('com_ganewsletter.edit.newsletter.id', null);
                $app->setUserState('com_ganewsletter.edit.newsletter.data', null);

                $app->enqueueMessage(Text::_('COM_GANEWSLETTER_GEN_PDF_SUCCESSFULLY'), 'success');

			}

		} else {
			throw new \Exception(500);
		}

		// Redirect to the list screen.
		$app->redirect(Route::_('index.php?option=com_ganewsletter&view=newsletters', false));

	}

	/**
	 * Send out the Newsletter
	 * @return void
	 * @throws Exception
	 */
	public function sendNews()
	{
		// Initialise variables.
		$app = Factory::getApplication();

		// Get the user submitted data.
		$id = $app->input->getInt('id', 0);
		$item = GanewsletterHelper::getNewsletterRecord($id);

		// Checking if the user can remove object
		$user = GanewsletterHelper::getSpecificUser();
		$canEdit = GanewsletterHelper::canUserEdit($user, $item);

		if ($canEdit)
		{
			$model = $this->getModel('Newsletter', 'Site');

			// Attempt to save the data.
			$return = $model->sendNews($id, $item);

			// Check for errors.
			if ($return === false) {
				$this->setMessage(Text::sprintf(Text::_('COM_GANEWSLETTER_SEND_FAILED'), $model->getError()), 'warning');
			} else {
				// Check in the record.
				if ($return) {
					$model->checkin($return);
				}

                $app->setUserState('com_ganewsletter.edit.newsletter.id', null);
                $app->setUserState('com_ganewsletter.edit.newsletter.data', null);

                $app->enqueueMessage(Text::_('COM_GANEWSLETTER_SEND_SUCCESSFULLY'), 'success');

			}

		} else {
			throw new \Exception(500);
		}

		// Redirect to the list screen.
		$app->redirect(Route::_('index.php?option=com_ganewsletter&view=newsletters', false));

	}
}
